// @@@ START COPYRIGHT @@@
//
// (C) Copyright 2000-2014 Hewlett-Packard Development Company, L.P.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.
//
// @@@ END COPYRIGHT @@@

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wchar.h>
#include <math.h>
#include <limits.h>
#include <fstream> 
#include "sqlcli.h"
#include "sqludr.h"

using namespace tmudr;

// C++ QATmudf
// Note: This code is the same as shown in the tutorial on the wiki
// https://wiki.trafodion.org/wiki/index.php/Tutorial:_The_object-oriented_UDF_interface

class QATmudf : public UDRInterface
{
  // override any methods where the UDF author would
  // like to change the default behavior
  void describeParamsAndColumns(UDRInvocationInfo &info); // Binder
  void processData(UDRInvocationInfo &info,
                   UDRPlanInfo &plan);
};

extern "C" UDRInterface * QA_TMUDF(
     const UDRInvocationInfo *info)
{
  return new QATmudf();
}

void QATmudf::describeParamsAndColumns(UDRInvocationInfo &info)
{
  // do this logic only if the UDF name matches
  if (! info.getUDRName().find(".QaTmUdf"))
  {
    throw UDRException(
         38086,
         "Called compiler interface for TMUDF QaTMUdf for TMUDF %s",
         info.getUDRName().data());
  }

  // sessionize is intended to work with a single input table
  if (info.getNumTableInputs() != 1)
    throw UDRException(38085,
                       "Expecting one table-valued input, got %d",
                       info.getNumTableInputs());

  // remove any output columns declared in the DDL
  while (info.getOutputTableInfo().getNumColumns() > 0)
    info.getOutputTableInfo().deleteColumn(0);

  // add all input table columns as output columns (t is the entire table)
  for (int t=0; t<info.getNumTableInputs(); t++)
  {
    info.addPassThruColumns(t);
  }
}

void QATmudf::processData(UDRInvocationInfo &info,
                                UDRPlanInfo &plan)
{
  // loop over input rows and return the same rows
  while (getNextRow(info))
  {
    // copy all columns and emit the row
    info.copyPassThruData();

    emitRow(info);
  }
}

